/* uname -- print system information
   Copyright (C) 89, 90, 91, 92, 93, 1994 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* Option		Example

   -s, --sysname	SunOS
   -n, --nodename	rocky8
   -r, --release	4.0
   -v, --version	
   -m, --machine	sun
   -p, --processor	
   -a, --all		SunOS rocky8 4.0  sun

   The default behavior is equivalent to `-s'.

   David MacKenzie <djm@gnu.ai.mit.edu> */

/* Ported to NEWS-OS by Yoshinari KANAYA <kanaya@yumimi.dais.is.tohoku.ac.jp> */

#if !defined(lint)
static char *rcsid = "$Id: uname.c,v 1.1 1995/07/18 13:24:36 kanaya Exp kanaya $";
#endif /* ! lint */

#include <stdio.h>
#include <strings.h>
#include <news/sysnews.h>
#include "getopt.h"
#include "utsname.h"

static char *version_string = "uname for NEWS-OS 4.x based on GNU sh-utils 1.12";

static void print_element();
static void usage();

/* Values that are bitwise or'd into `toprint'. */
#define PRINT_SYSNAME	1	/* Operating system name. */
#define PRINT_NODENAME	2	/* Node name on a communications network. */
#define PRINT_RELEASE	4	/* Operating system release. */
#define PRINT_VERSION	8	/* Operating system version. */
#define PRINT_MACHINE	16	/* Machine hardware name. */
#define PRINT_PROCESSOR	32	/* Current host's processor type. */

/* Mask indicating which elements of the name to print. */
static unsigned char toprint;

/* The name this program was run with, for error messages. */
static char *program_name;

/* If non-zero, display usage information and exit. */
static int show_help;

/* If non-zero, print the version on standard output and exit. */
static int show_version;

static struct option long_options[] =
{
	{"help", no_argument, &show_help, 1},
	{"machine", no_argument, NULL, 'm'},
	{"nodename", no_argument, NULL, 'n'},
	{"processor", no_argument, NULL, 'p'},
	{"release", no_argument, NULL, 'r'},
	{"sysname", no_argument, NULL, 's'},
	{"version", no_argument, &show_version, 1},
	{"all", no_argument, NULL, 'a'},
	{NULL, 0, NULL, 0}
};

void
main (argc, argv)
	int argc;
	char **argv;
{
	struct utsname name;
	struct machtype machtype_buf;
	char news_version[BUFSIZ], str[BUFSIZ];
	int c;

	program_name = argv[0];
	toprint = 0;

	while ((c = getopt_long(argc, argv, "snrvmpa", long_options, (int *) 0))
	       != EOF)
	{
		switch (c) {
		case 0:
			break;

		case 's':
			toprint |= PRINT_SYSNAME;
			break;

		case 'n':
			toprint |= PRINT_NODENAME;
			break;

		case 'r':
			toprint |= PRINT_RELEASE;
			break;

		case 'v':
			toprint |= PRINT_VERSION;
			break;

		case 'm':
			toprint |= PRINT_MACHINE;
			break;

		case 'p':
			toprint |= PRINT_PROCESSOR;
			break;

		case 'a':
			toprint = PRINT_SYSNAME | PRINT_NODENAME |
			          PRINT_RELEASE | PRINT_VERSION  |
			          PRINT_MACHINE | PRINT_PROCESSOR;
			break;

		default:
			usage(1);
		}
	}

	if (show_version) {
		printf("uname - %s\n", version_string);
		exit(0);
	}

	if (show_help)
		usage(0);

	if (optind != argc)
		usage(1);

	if (toprint == 0)
		toprint = PRINT_SYSNAME;

	if (gethostname(name.nodename, sizeof(name.nodename)) == -1) {
		perror(program_name);
		exit(1);
	}

	if (sysnews(NEWS_VERSION, news_version, sizeof(news_version)) == -1) {
		perror(program_name);
		exit(1);
	}

	if (sscanf(news_version, "%s Release %[^:]", name.sysname, str) != 2) {
		fprintf(stderr, "cannot get system name");
		exit(1);
	} else {
		char *p;

		if ((p = rindex(str, '#')) == NULL) {
			fprintf(stderr, "cannot get version");
			exit(1);
		}
		strcpy(name.version, p);
		*(--p) = '\0';
		strcpy(name.release, str);
	}

	if (sysnews(NEWS_MACHTYPE, &machtype_buf) == -1) {
		perror(program_name);
		exit(1);
	}

	strcpy(name.machine, machtype_buf.mt_machine);
	strcpy(name.processor, machtype_buf.mt_maincpu);

	print_element(PRINT_SYSNAME, name.sysname);
	print_element(PRINT_NODENAME, name.nodename);
	print_element(PRINT_RELEASE, name.release);
	print_element(PRINT_VERSION, name.version);
	print_element(PRINT_MACHINE, name.machine);
	print_element(PRINT_PROCESSOR, name.processor);

	exit(0);
}

/* If the name element set in MASK is selected for printing in `toprint',
   print ELEMENT; then print a space unless it is the last element to
   be printed, in which case print a newline. */

static void
print_element(mask, element)
	unsigned char mask;
	char *element;
{
	if (toprint & mask) {
		toprint &= ~mask;
		printf("%s%c", element, toprint ? ' ' : '\n');
	}
}

static void
usage(status)
	int status;
{
	if (status != 0)
		fprintf(stderr, "Try `%s --help' for more information.\n",
		        program_name);
	else {
		printf("Usage: %s [OPTION]...\n", program_name);
		printf("\
\n\
  -a, --all        print all information\n\
  -m, --machine    print the machine (hardware) type\n\
  -n, --nodename   print the machine's network node hostname\n\
  -p, --processor  print the current host's processor type\n\
  -r, --release    print the operating system release\n\
  -s, --sysname    print the operating system name\n\
  -v               print the operating system version\n\
      --help       display this help and exit\n\
      --version    output version information and exit\n\
\n\
Without any OPTION, assume -s.\n\
");
	}
	exit(status);
}
